//
//
// hello.array.44.2.c
//
// Charlieplex LED array hello-world
// two-layer version
//
// Neil Gershenfeld
// 11/11/11
// Versión Adolfo Benítez
// 2376/2015
// Signal input


#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input 
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define led_delay() _delay_ms(1) // LED delay

#define led_port PORTA
#define led_direction DDRA

#define A (1 << PA0) // row 1
#define B (1 << PA1) // row 2
#define C (1 << PA2) // row 3
#define D (1 << PA5) // row 4 (PA3 it is broken)
#define E (1 << PA4) // row 5

void flash(uint8_t from, uint8_t to, uint8_t delay) {
   //
   // source from, sink to, flash
   //
   static uint8_t i;
   set(led_port,from);
   clear(led_port,to);
   output(led_direction,from);
   output(led_direction,to);
   for (i = 0; i < delay; ++i)
       led_delay();
   input(led_direction,from);
   input(led_direction,to);
   }

void led_cycle_low(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {

	flash(D,E,delay);
	flash(E,D,delay);
	flash(E,C,delay);
	flash(E,B,delay);
	flash(E,A,delay);

      }
   }

void led_cycle_target1(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      //flash(B,A,delay);
      //flash(C,A,delay);
      flash(D,A,delay);
      flash(E,A,delay);
      //flash(A,B,delay);
      //flash(C,B,delay);
      flash(D,B,delay);
      flash(E,B,delay);
      //flash(A,C,delay);
      //flash(B,C,delay);
      flash(D,C,delay);
      flash(E,C,delay);
      //flash(A,D,delay);
      //flash(B,D,delay);
      flash(C,D,delay);
      flash(E,D,delay);
      //flash(A,E,delay);
      //flash(B,E,delay);
      flash(C,E,delay);
      flash(D,E,delay);
      }
   }

void led_cycle_target2(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      //flash(B,A,delay);
      flash(C,A,delay);
      flash(D,A,delay);
      flash(E,A,delay);
      //flash(A,B,delay);
      flash(C,B,delay);
      flash(D,B,delay);
      flash(E,B,delay);
      //flash(A,C,delay);
      flash(B,C,delay);
      flash(D,C,delay);
      flash(E,C,delay);
      //flash(A,D,delay);
      flash(B,D,delay);
      flash(C,D,delay);
      flash(E,D,delay);
      //flash(A,E,delay);
      flash(B,E,delay);
      flash(C,E,delay);
      flash(D,E,delay);
      }
   }

void led_cycle_slide1(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      flash(B,A,delay);
      flash(C,A,delay);
      flash(D,A,delay);
      flash(E,A,delay);
      //flash(A,B,delay);
      //flash(C,B,delay);
      flash(D,B,delay);
      //flash(E,B,delay);
      //flash(A,C,delay);
      flash(B,C,delay);
      //flash(D,C,delay);
      //flash(E,C,delay);
      flash(A,D,delay);
      //flash(B,D,delay);
      //flash(C,D,delay);
      //flash(E,D,delay);
      flash(A,E,delay);
      flash(B,E,delay);
      flash(C,E,delay);
      flash(D,E,delay);
      }
   }

void led_cycle_slide2(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      //flash(B,A,delay);
      //flash(C,A,delay);
      //flash(D,A,delay);
      //flash(E,A,delay);
      flash(A,B,delay);
      flash(C,B,delay);
      flash(D,B,delay);
      flash(E,B,delay);
      //flash(A,C,delay);
      //flash(B,C,delay);
      //flash(D,C,delay);
      flash(E,C,delay);
      //flash(A,D,delay);
      //flash(B,D,delay);
      flash(C,D,delay);
      //flash(E,D,delay);
      //flash(A,E,delay);
      flash(B,E,delay);
      //flash(C,E,delay);
      //flash(D,E,delay);
      }
   }

void led_cycle_slide3(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      //flash(B,A,delay);
      flash(C,A,delay);
      flash(D,A,delay);
      //flash(E,A,delay);
      
      //flash(A,B,delay);
      //flash(C,B,delay);
      //flash(D,B,delay);
      //flash(E,B,delay);

      flash(A,C,delay);
      flash(B,C,delay);
      flash(D,C,delay);
      flash(E,C,delay);

      //flash(A,D,delay);
      //flash(B,D,delay);
      //flash(C,D,delay);
      flash(E,D,delay);

      //flash(A,E,delay);
      //flash(B,E,delay);
      flash(C,E,delay);
      //flash(D,E,delay);
      }
   }


void led_cycle_slide4(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      flash(B,A,delay);
      //flash(C,A,delay);
      //flash(D,A,delay);
      flash(E,A,delay);

      //flash(A,B,delay);
      flash(C,B,delay);
      flash(D,B,delay);
      //flash(E,B,delay);

      //flash(A,C,delay);
      //flash(B,C,delay);
      //flash(D,C,delay);
      //flash(E,C,delay);

      flash(A,D,delay);
      flash(B,D,delay);
      flash(C,D,delay);
      flash(E,D,delay);

      //flash(A,E,delay);
      //flash(B,E,delay);
      flash(C,E,delay);
      //flash(D,E,delay);
      }
   }


void led_cycle_slide5(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      flash(B,A,delay);
      //flash(C,A,delay);
      //flash(D,A,delay);
      flash(E,A,delay);

      flash(A,B,delay);
      //flash(C,B,delay);
      //flash(D,B,delay);
      flash(E,B,delay);

      //flash(A,C,delay);
      flash(B,C,delay);
      flash(D,C,delay);
      //flash(E,C,delay);

      //flash(A,D,delay);
      //flash(B,D,delay);
      //flash(C,D,delay);
      //flash(E,D,delay);

      flash(A,E,delay);
      flash(B,E,delay);
      flash(C,E,delay);
      flash(D,E,delay);
      }
   }


void led_cycle_slide6(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      flash(B,A,delay);
      //flash(C,A,delay);
      //flash(D,A,delay);
      flash(E,A,delay);

      flash(A,B,delay);
      //flash(C,B,delay);
      //flash(D,B,delay);
      flash(E,B,delay);

      flash(A,C,delay);
      //flash(B,C,delay);
      //flash(D,C,delay);
      flash(E,C,delay);

      //flash(A,D,delay);
      flash(B,D,delay);
      flash(C,D,delay);
      //flash(E,D,delay);

      //flash(A,E,delay);
      //flash(B,E,delay);
      //flash(C,E,delay);
      //flash(D,E,delay);
      }
   }


void led_cycle_slide7(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      //flash(B,A,delay);
      flash(C,A,delay);
      flash(D,A,delay);
      //flash(E,A,delay);

      flash(A,B,delay);
      //flash(C,B,delay);
      //flash(D,B,delay);
      flash(E,B,delay);

      flash(A,C,delay);
      //flash(B,C,delay);
      //flash(D,C,delay);
      flash(E,C,delay);

      flash(A,D,delay);
      //flash(B,D,delay);
      //flash(C,D,delay);
      flash(E,D,delay);

      //flash(A,E,delay);
      flash(B,E,delay);
      flash(C,E,delay);
      //flash(D,E,delay);
      }
   }

int main(void) {
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // main loop
   //
   while (1) {
	led_cycle_low(10,70);
	led_cycle_target1(200,1);
	led_cycle_target1(200,1);
	led_cycle_target2(200,1);
	led_cycle_target2(200,1);

	led_cycle_slide1(200,1);
	led_cycle_slide2(25,1);
	led_cycle_slide3(25,1);
	led_cycle_slide4(25,1);
	led_cycle_slide5(25,1);
	led_cycle_slide6(25,1);
	led_cycle_slide7(100,1);

	led_cycle_slide1(200,1);
	led_cycle_slide2(25,1);
	led_cycle_slide3(25,1);
	led_cycle_slide4(25,1);
	led_cycle_slide5(25,1);
	led_cycle_slide6(25,1);
	led_cycle_slide7(100,1);

	led_cycle_slide1(200,1);
	led_cycle_slide2(25,1);
	led_cycle_slide3(25,1);
	led_cycle_slide4(25,1);
	led_cycle_slide5(25,1);
	led_cycle_slide6(25,1);
	led_cycle_slide7(100,1);
 
      }
   }
